@@ -33,6 +33,13 @@ class InvalidArgsError(Exception):
     pass
 
 
+class RequiredPackageError(Exception):
+
+    def __str__(self):
+        return "%s is required, please add a --package declaration for it, " \
+               "or its parameters, to the JCC invocation." %(self.args[0])
+
+
 _jcc3._set_exception_types(JavaError, InvalidArgsError)
 from ._jcc3 import findClass as _findClass
 from ._jcc3 import *
@@ -52,6 +59,65 @@ def findClass(className):
     return cls
 
 
+# fix for PYLUCENE-47: method overloads need to be sorted deterministically
+# and heuristically by the so-called rank of their parameter types, trying
+# to match parameters whose classes are deeper in the class hierarchy first
+def sortSignatures(signatures, groupByName=False):
+    class Parameters(object):
+        def __init__(self, signature):
+            self.signature = signature
+            self.types = signature.getParameterTypes()
+            if groupByName:
+                self.name = signature.getName()
+
+        def __lt__(self, other):
+            # group signatures by name, lexicographically
+            if groupByName:
+                if self.name < other.name:
+                    return True
+                if self.name > other.name:
+                    return False
+
+            # group signatures by number of parameters, lowest first
+            diff = len(self.types) - len(other.types)
+            if diff < 0:
+                return True
+            if diff > 0:
+                return False
+
+            # sort signatures of equal length by the sum of class ranks
+            # so that deepest classes are considered first during calls,
+            # thus sort highest rank number first
+            diff = self.getRank() - other.getRank()
+            if diff > 0:
+                return True
+            if diff < 0:
+                return False
+
+            # sort signatures of equal rank by the names of their classes
+            # lexicographically
+            for (t_self, t_other) in zip(self.types, other.types):
+                if t_self.getName() < t_other.getName():
+                    return True
+
+            # give up, other must be larger ;-)
+            return False
+            
+        def getRank(self):
+            rank = 0
+            for item in self.types:
+                while item is not None:
+                    rank += 1
+                    item = item.getSuperclass()
+
+            return rank
+
+    parameters = [Parameters(signature) for signature in signatures]
+    parameters.sort()
+
+    signatures[:] = [params.signature for params in parameters]
+
+
 INDENT = '  '
 HALF_INDENT = ' '
 
@@ -68,7 +134,8 @@ PRIMITIVES = { 'boolean': 'Z',
 RESERVED = set(['delete', 'and', 'or', 'not', 'xor', 'union', 'register',
                 'const', 'bool', 'operator', 'typeof', 'asm', 'mutable',
                 'inline', 'typedef', 'struct', 'extern',
-                'NULL', 'DOMAIN', 'IGNORE', 'min', 'max', 'PREFIX', 'NAN'])
+                'NULL', 'DOMAIN', 'IGNORE', 'min', 'max', 'PREFIX', 'NAN',
+                'DEBUG'])
 
 RENAME_METHOD_SUFFIX = '_'
 RENAME_FIELD_SUFFIX = '__'
@@ -380,6 +447,7 @@ def jcc(args):
     use_distutils = False
     shared = False
     dist = False
+    wheel = False
     wininst = False
     find_jvm_dll = False
     compiler = None
@@ -487,6 +555,10 @@ def jcc(args):
             elif arg == '--bdist':
                 from .python import compile
                 dist = True
+            elif arg == '--wheel':
+                from .python import compile
+                wheel = True
+                dist = True
             elif arg == '--wininst':
                 from .python import compile
                 wininst = True
@@ -526,17 +598,20 @@ def jcc(args):
     initvm_args['maxstack'] = '512k'
     initvm_args['vmargs'] = vmargs
 
-    env = initVM(os.pathsep.join(classpath) or None, **initvm_args)
-
-    typeset = set()
-    excludes = set(excludes)
-
     if imports:
         if shared:
             imports = dict((__import__(import_), set()) for import_ in imports)
         else:
             raise ValueError("--shared must be used when using --import")
 
+        for import_ in imports.keys():
+            classpath.append(import_.CLASSPATH)
+
+    env = initVM(os.pathsep.join(classpath) or None, **initvm_args)
+
+    typeset = set()
+    excludes = set(excludes)
+
     if recompile or not build and (install or dist or egg_info):
         if moduleName is None:
             raise ValueError('module name not specified (use --python)')
@@ -544,24 +619,21 @@ def jcc(args):
             compile(env, os.path.dirname(args[0]), output, moduleName,
                     install, dist, debug, jars, version,
                     prefix, root, install_dir, home_dir, use_distutils,
-                    shared, compiler, modules, wininst, find_jvm_dll,
+                    shared, compiler, modules, wheel, wininst, find_jvm_dll,
                     arch, generics, resources, imports, use_full_names,
                     egg_info, extra_setup_args)
     else:
         if imports:
-            def walk(args, dirname, names):
-                (include, importset) = args
-                for name in names:
-                    if name.endswith('.h'):
-                        className = os.path.join(dirname[len(include) + 1:],
-                                                 name[:-2])
-                        if os.path.sep != '/':
-                            className = className.replace(os.path.sep, '/')
-                        importset.add(findClass(className))
             for import_, importset in imports.items():
-                env._addClassPath(import_.CLASSPATH)
                 include = os.path.join(import_.__module_dir__, 'include')
-                os.path.walk(include, walk, (include, importset))
+                for dirpath, dirnames, filenames in os.walk(include):
+                    for name in filenames:
+                        if name.endswith('.h'):
+                            className = os.path.join(dirpath[len(include) + 1:],
+                                                     name[:-2])
+                            if os.path.sep != '/':
+                                className = className.replace(os.path.sep, '/')
+                            importset.add(findClass(className))
                 typeset.update(importset)
         typeset.add(findClass('java/lang/Object'))
         typeset.add(findClass('java/lang/Class'))
@@ -703,7 +775,7 @@ def jcc(args):
                 compile(env, os.path.dirname(args[0]), output, moduleName,
                         install, dist, debug, jars, version,
                         prefix, root, install_dir, home_dir, use_distutils,
-                        shared, compiler, modules, wininst, find_jvm_dll,
+                        shared, compiler, modules, wheel, wininst, find_jvm_dll,
                         arch, generics, resources, imports, use_full_names,
                         egg_info, extra_setup_args)
 
@@ -733,6 +805,9 @@ def header(env, out, cls, typeset, packages, excludes, generics,
                 interfaces.append(interface)
                 if pt is not None:
                     for ta in pt.getActualTypeArguments():
+                        if not known(ta, typeset, declares, packages,
+                                     excludes, True):
+                            raise RequiredPackageError(ta)
                         addRequiredTypes(ta, typeset, True)
     else:
         for interface in cls.getInterfaces():
@@ -753,8 +828,10 @@ def header(env, out, cls, typeset, packages, excludes, generics,
         if generics:
             for clsParam in getTypeParameters(cls):
                 if Class.instance_(clsParam):
+                    if not known(clsParam, typeset, declares, packages,
+                                 excludes, True):
+                        raise RequiredPackageError(clsParam)
                     addRequiredTypes(clsParam, typeset, True)
-                    known(clsParam, typeset, declares, packages, excludes, True)
     else:
         superClsNames = ['JObject']
 
@@ -788,7 +865,7 @@ def header(env, out, cls, typeset, packages, excludes, generics,
                     break
             else:
                 constructors.append(constructor)
-    constructors.sort(key=lambda x: len(x.getParameterTypes()))
+    sortSignatures(constructors)
 
     methods = {}
     protectedMethods = []
@@ -825,7 +902,7 @@ def header(env, out, cls, typeset, packages, excludes, generics,
             protectedMethods.append(method)
 
     methods = list(methods.values())
-    methods.sort(key=lambda x: (x.getName(), len(x.getParameterTypes())))
+    sortSignatures(methods, groupByName=True)
     methodNames = set([cppname(method.getName()) for method in methods])
 
     for constructor in constructors:
