@@ -18,7 +18,7 @@ from . import _jcc3
 from .cpp import PRIMITIVES, INDENT, HALF_INDENT
 from .cpp import RENAME_METHOD_SUFFIX, RENAME_FIELD_SUFFIX
 from .cpp import cppname, cppnames, absname, typename, findClass
-from .cpp import line, signature, find_method
+from .cpp import line, signature, find_method, sortSignatures
 from .cpp import Modifier, Class, Method
 from .cpp import getActualTypeArguments, getTypeParameters
 from .config import INCLUDES, CFLAGS, DEBUG_CFLAGS, LFLAGS, IMPLIB_LFLAGS, \
@@ -734,7 +734,7 @@ def python(env, out_h, out, cls, superClasses, names, superNames,
 
     for name, methods in allMethods:
         args, x, cardinality = methodargs(methods, superMethods)
-        methods.sort(key=lambda x: len(x.getParameterTypes()))
+        sortSignatures(methods)
         method = methods[0]
         modifiers = method.getModifiers()
         if name == 'iterator' and iteratorMethod is None:
@@ -772,7 +772,7 @@ def python(env, out_h, out, cls, superClasses, names, superNames,
 
     for name, methods in extMethods:
         args, x, cardinality = methodargs(methods, superMethods)
-        methods.sort(key=lambda x: len(x.getParameterTypes()))
+        sortSignatures(methods)
         method = methods[0]
         modifiers = method.getModifiers()
         if name == 'iterator' and iteratorMethod is None:
@@ -1423,7 +1423,7 @@ def package(out, allInOne, cppdir, namespace, names, use_full_names):
         line(out, 0, '#include "JCCEnv.h"')
         line(out, 0, '#include "functions.h"')
 
-    if not names:
+    if not names or not allInOne:
         line(out)
         line(out, 0, 'PyObject *initVM(PyObject *module, PyObject *args, PyObject *kwds);')
         if use_full_names:
@@ -1590,8 +1590,9 @@ def module(out, allInOne, classes, imports, cppdir, moduleName,
 
 def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
             version, prefix, root, install_dir, home_dir, use_distutils,
-            shared, compiler, modules, wininst, find_jvm_dll, arch, generics,
-            resources, imports, use_full_names, egg_info, extra_setup_args):
+            shared, compiler, modules, wheel, wininst, find_jvm_dll, arch,
+            generics, resources, imports, use_full_names, egg_info,
+            extra_setup_args):
     try:
         if use_distutils:
             raise ImportError
@@ -1801,7 +1802,7 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         if wininst:
             script_args.append('bdist_wininst')
         elif with_setuptools:
-            script_args.append('bdist_egg')
+            script_args.append('bdist_wheel' if wheel else 'bdist_egg')
         else:
             script_args.append('bdist')
 
@@ -1869,6 +1870,7 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
                 i += 1
         config_vars['CFLAGS'] = ' '.join(cflags)
 
+    print("extension args = %s" % args)
     extensions = [Extension('.'.join([moduleName, extname]), **args)]
     script_args.extend(extra_setup_args)
 
@@ -1877,10 +1879,11 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         'packages': packages,
         'package_dir': {moduleName: modulePath},
         'package_data': {moduleName: package_data},
-        'version': version,
         'ext_modules': extensions,
         'script_args': script_args
     }
+    if version:
+        args['version'] = version
     if with_setuptools:
         args['zip_safe'] = False
 
