@@ -537,8 +537,7 @@ int _parseArgs(PyObject **args, unsigned int count, char *types, ...)
 
                   if (last)
                   {
-                      if ((PyBytes_Check(arg) && (PyBytes_Size(arg) == 1)) ||
-                          PyLong_CheckExact(arg))
+                      if (PyBytes_Check(arg) && (PyBytes_Size(arg) == 1))
                       {
                           varargs = true;
                           break;
@@ -547,8 +546,6 @@ int _parseArgs(PyObject **args, unsigned int count, char *types, ...)
               }
               else if (PyBytes_Check(arg) && (PyBytes_Size(arg) == 1))
                   break;
-              else if (PyLong_CheckExact(arg))
-                  break;
 
               return -1;
           }
@@ -1711,14 +1708,24 @@ PyTypeObject *makeType(PyType_Def *def)
             count += 1;
 
         PyObject *bases = PyTuple_New(count);
+        int basicsize = def->spec.basicsize;
 
-        for (int i = 0; i < count; ++i)
+        for (int i = 0; i < count; ++i) {
             PyTuple_SET_ITEM(bases, i, (PyObject *) makeType(def->bases[i]));
+            if (basicsize < def->bases[i]->spec.basicsize)
+                basicsize = def->bases[i]->spec.basicsize;
+        }
 
+        def->spec.basicsize = basicsize;  // or typeobject.c, line 2092, fails
         def->type = (PyTypeObject *)
             PyType_FromSpecWithBases(&def->spec, bases);
+
         Py_DECREF(bases);
     }
+    else
+    {
+        Py_INCREF((PyObject *) def->type);
+    }
 
     return def->type;
 }
@@ -1729,7 +1736,6 @@ void installType(PyTypeObject **type, PyType_Def *def,
     if (*type == NULL)
     {
         *type = makeType(def);
-        Py_INCREF(*type);
 
         if (isExtension)
         {
@@ -1737,6 +1743,14 @@ void installType(PyTypeObject **type, PyType_Def *def,
             Py_INCREF(PY_TYPE(FinalizerClass));
         }
 
+        PyObject *module_name = PyModule_GetNameObject(module);
+
+        if (module_name != NULL)
+        {
+            PyDict_SetItemString((*type)->tp_dict, "__module__", module_name);
+            Py_DECREF(module_name);
+        }
+
         PyModule_AddObject(module, name, (PyObject *) *type);
     }
 }
