@@ -12,7 +12,7 @@
 
 import os, sys, platform, subprocess
 
-jcc_ver = '3.10'
+jcc_ver = '3.11'
 machine = platform.machine()
 using_python2 = sys.version_info < (3,)
 
@@ -33,7 +33,7 @@ else:
 # Add or edit the entry corresponding to your system in the JDK, INCLUDES,
 # CFLAGS, DEBUG_CFLAGS, LFLAGS and JAVAC dictionaries below.
 # These entries are used to build JCC _and_ by JCC to drive compiling and
-# linking via distutils or setuptools the extensions it generated code for.
+# linking via distutils or setuptools the extensions it generates code for.
 #
 # The key for your system is determined by the platform variable defined
 # above.
@@ -42,6 +42,9 @@ else:
 # dictionaries with JCC_JDK, JCC_INCLUDES, JCC_CFLAGS, JCC_DEBUG_CFLAGS,
 # JCC_LFLAGS and JCC_JAVAC environment variables using os.pathsep as value
 # separator.
+#
+# On macos (darwin), the JAVA_HOME environment variable may be used to
+# override the /usr/libexec/java_home program.
 
 if platform in ("win32", "mingw32"):
     try:
@@ -99,6 +102,8 @@ running setup.py.
 
 INCLUDES = {
     'darwin/frameworks': ['%(darwin)s/Headers' %(JDK)],
+    'darwin/temurin': ['%(darwin)s/include' %(JDK),
+                       '%(darwin)s/include/darwin' %(JDK)],
     'darwin/adoptopenjdk': ['%(darwin)s/include' %(JDK),
                             '%(darwin)s/include/darwin' %(JDK)],
     'darwin/home': ['%(darwin)s/include' %(JDK),
@@ -141,15 +146,20 @@ DEBUG_CFLAGS = {
 
 LFLAGS = {
     'darwin/frameworks': ['-framework', 'JavaVM', '-mmacosx-version-min=10.9'],
+    'darwin/temurin': ['-L%(darwin)s/lib' %(JDK), '-ljava',
+                       '-L%(darwin)s/lib/server' %(JDK), '-ljvm',
+                       '-Wl,-rpath', '-Wl,%(darwin)s/lib' %(JDK),
+                       '-Wl,-rpath', '-Wl,%(darwin)s/lib/server' %(JDK),
+                       '-mmacosx-version-min=11.6'],
     'darwin/adoptopenjdk': ['-L%(darwin)s/lib' %(JDK), '-ljava',
                             '-L%(darwin)s/lib/server' %(JDK), '-ljvm',
                             '-Wl,-rpath', '-Wl,%(darwin)s/lib' %(JDK),
                             '-Wl,-rpath', '-Wl,%(darwin)s/lib/server' %(JDK),
                             '-mmacosx-version-min=10.9'],
-    'darwin/home': ['-L%(darwin)s/jre/lib' %(JDK), '-ljava',
-                    '-L%(darwin)s/jre/lib/server' %(JDK), '-ljvm',
-                    '-Wl,-rpath', '-Wl,%(darwin)s/jre/lib' %(JDK),
-                    '-Wl,-rpath', '-Wl,%(darwin)s/jre/lib/server' %(JDK),
+    'darwin/home': ['-L%(darwin)s/lib' %(JDK), '-ljava',
+                    '-L%(darwin)s/lib/server' %(JDK), '-ljvm',
+                    '-Wl,-rpath', '-Wl,%(darwin)s/lib' %(JDK),
+                    '-Wl,-rpath', '-Wl,%(darwin)s/lib/server' %(JDK),
                     '-mmacosx-version-min=10.9'],
     'ipod': ['-ljvm', '-lpython%s.%s' %(sys.version_info[0:2]),
              '-L/usr/lib/gcc/arm-apple-darwin9/4.0.1'],
@@ -181,7 +191,10 @@ if platform == 'linux':
     LFLAGS['linux'] = LFLAGS['linux/%s' %(machine)]
 elif platform == 'darwin':
     if JAVAHOME is not None:
-        if 'adoptopenjdk' in JAVAHOME:
+        if 'temurin' in JAVAHOME:
+            INCLUDES['darwin'] = INCLUDES['darwin/temurin']
+            LFLAGS['darwin'] = LFLAGS['darwin/temurin']
+        elif 'adoptopenjdk' in JAVAHOME:
             INCLUDES['darwin'] = INCLUDES['darwin/adoptopenjdk']
             LFLAGS['darwin'] = LFLAGS['darwin/adoptopenjdk']
         else:
