@@ -519,7 +519,7 @@ public class TestRandomChains extends BaseTokenStreamTestCase {
         return DateFormat.getDateInstance(DateFormat.DEFAULT, randomLocale(random));
     });
     put(Automaton.class, random -> {
-        return Operations.determinize(new RegExp(AutomatonTestUtil.randomRegexp(random), RegExp.NONE).toAutomaton(), Operations.DEFAULT_MAX_DETERMINIZED_STATES);
+        return Operations.determinize(new RegExp(AutomatonTestUtil.randomRegexp(random), RegExp.NONE).toAutomaton(), Operations.DEFAULT_DETERMINIZE_WORK_LIMIT);
     });
     put(
         PatternTypingFilter.PatternTypingRule[].class,
