@@ -126,6 +126,7 @@ public final class KoreanTokenizer extends Tokenizer {
   private final FST.BytesReader userFSTReader;
   private final TokenInfoFST userFST;
 
+  private final boolean discardPunctuation;
   private final DecompoundMode mode;
   private final boolean outputUnknownUnigrams;
 
@@ -159,27 +160,68 @@ public final class KoreanTokenizer extends Tokenizer {
    * Uses the default AttributeFactory.
    */
   public KoreanTokenizer() {
-    this(DEFAULT_TOKEN_ATTRIBUTE_FACTORY, null, DEFAULT_DECOMPOUND, false);
+    this(DEFAULT_TOKEN_ATTRIBUTE_FACTORY, null, DEFAULT_DECOMPOUND, false, true);
   }
 
   /**
-   * Create a new KoreanTokenizer.
+   * Create a new KoreanTokenizer using the system and unknown dictionaries shipped with Lucene.
    *
    * @param factory the AttributeFactory to use
    * @param userDictionary Optional: if non-null, user dictionary.
    * @param mode Decompound mode.
-   * @param outputUnknownUnigrams If true outputs unigrams for unknown words.
+   * @param outputUnknownUnigrams if true outputs unigrams for unknown words.
    */
   public KoreanTokenizer(AttributeFactory factory, UserDictionary userDictionary, DecompoundMode mode, boolean outputUnknownUnigrams) {
+    this(factory, userDictionary, mode, outputUnknownUnigrams, true);
+  }
+
+  /**
+   * Create a new KoreanTokenizer using the system and unknown dictionaries shipped with Lucene.
+   *
+   * @param factory the AttributeFactory to use
+   * @param userDictionary Optional: if non-null, user dictionary.
+   * @param mode Decompound mode.
+   * @param outputUnknownUnigrams if true outputs unigrams for unknown words.
+   * @param discardPunctuation true if punctuation tokens should be dropped from the output.
+   */
+  public KoreanTokenizer(AttributeFactory factory, UserDictionary userDictionary, DecompoundMode mode, boolean outputUnknownUnigrams, boolean discardPunctuation) {
+    this(factory,
+        TokenInfoDictionary.getInstance(),
+        UnknownDictionary.getInstance(),
+        ConnectionCosts.getInstance(),
+        userDictionary, mode, outputUnknownUnigrams, discardPunctuation);
+  }
+
+  /**
+   * <p>Create a new KoreanTokenizer supplying a custom system dictionary and unknown dictionary.
+   * This constructor provides an entry point for users that want to construct custom language models
+   * that can be used as input to {@link org.apache.lucene.analysis.ko.util.DictionaryBuilder}.</p>
+   *
+   * @param factory the AttributeFactory to use
+   * @param systemDictionary a custom known token dictionary
+   * @param unkDictionary a custom unknown token dictionary
+   * @param connectionCosts custom token transition costs
+   * @param userDictionary Optional: if non-null, user dictionary.
+   * @param mode Decompound mode.
+   * @param outputUnknownUnigrams if true outputs unigrams for unknown words.
+   * @param discardPunctuation true if punctuation tokens should be dropped from the output.
+   * @lucene.experimental
+   */
+  public KoreanTokenizer(AttributeFactory factory,
+                         TokenInfoDictionary systemDictionary,
+                         UnknownDictionary unkDictionary,
+                         ConnectionCosts connectionCosts,
+                         UserDictionary userDictionary,
+                         DecompoundMode mode,
+                         boolean outputUnknownUnigrams,
+                         boolean discardPunctuation) {
     super(factory);
-    this.mode = mode;
-    this.outputUnknownUnigrams = outputUnknownUnigrams;
-    dictionary = TokenInfoDictionary.getInstance();
-    fst = dictionary.getFST();
-    unkDictionary = UnknownDictionary.getInstance();
-    characterDefinition = unkDictionary.getCharacterDefinition();
+    this.dictionary = systemDictionary;
+    this.fst = dictionary.getFST();
+    this.unkDictionary = unkDictionary;
+    this.characterDefinition = unkDictionary.getCharacterDefinition();
+    this.costs = connectionCosts;
     this.userDictionary = userDictionary;
-    costs = ConnectionCosts.getInstance();
     fstReader = fst.getBytesReader();
     if (userDictionary != null) {
       userFST = userDictionary.getFST();
@@ -188,7 +230,9 @@ public final class KoreanTokenizer extends Tokenizer {
       userFST = null;
       userFSTReader = null;
     }
-
+    this.mode = mode;
+    this.outputUnknownUnigrams = outputUnknownUnigrams;
+    this.discardPunctuation = discardPunctuation;
     buffer.reset(this.input);
 
     resetState();
@@ -325,7 +369,7 @@ public final class KoreanTokenizer extends Tokenizer {
 
   }
 
-  private void add(Dictionary dict, Position fromPosData, int wordPos, int endPos, int wordID, Type type) throws IOException {
+  private void add(Dictionary dict, Position fromPosData, int wordPos, int endPos, int wordID, Type type) {
     final POS.Tag leftPOS = dict.getLeftPOS(wordID);
     final int wordCost = dict.getWordCost(wordID);
     final int leftID = dict.getLeftId(wordID);
@@ -514,16 +558,13 @@ public final class KoreanTokenizer extends Tokenizer {
     // Index of the last character of unknown word:
     int unknownWordEndIndex = -1;
 
+    // Maximum posAhead of user word in the entire input
+    int userWordMaxPosAhead = -1;
+
     // Advances over each position (character):
-    while (true) {
-
-      if (buffer.get(pos) == -1) {
-        // End
-        break;
-      }
-
+    while (buffer.get(pos) != -1) {
       final Position posData = positions.get(pos);
-      final boolean isFrontier = positions.getNextPos() == pos+1;
+      final boolean isFrontier = positions.getNextPos() == pos + 1;
 
       if (posData.count == 0) {
         // No arcs arrive here; move to next position:
@@ -567,9 +608,9 @@ public final class KoreanTokenizer extends Tokenizer {
         int leastIDX = -1;
         int leastCost = Integer.MAX_VALUE;
         Position leastPosData = null;
-        for(int pos2=pos;pos2<positions.getNextPos();pos2++) {
+        for (int pos2 = pos; pos2 < positions.getNextPos(); pos2++) {
           final Position posData2 = positions.get(pos2);
-          for(int idx=0;idx<posData2.count;idx++) {
+          for (int idx = 0; idx < posData2.count; idx++) {
             //System.out.println("    idx=" + idx + " cost=" + cost);
             final int cost = posData2.costs[idx];
             if (cost < leastCost) {
@@ -584,7 +625,7 @@ public final class KoreanTokenizer extends Tokenizer {
         assert leastIDX != -1;
 
         // Second pass: prune all but the best path:
-        for(int pos2=pos;pos2<positions.getNextPos();pos2++) {
+        for (int pos2 = pos; pos2 < positions.getNextPos(); pos2++) {
           final Position posData2 = positions.get(pos2);
           if (posData2 != leastPosData) {
             posData2.reset();
@@ -629,21 +670,15 @@ public final class KoreanTokenizer extends Tokenizer {
         System.out.println("    " + posData.count + " arcs in");
       }
 
-      // Move to the first character that is not a whitespace.
-      // The whitespaces are added as a prefix for the term that we extract,
-      // this information is then used when computing the cost for the term using
-      // the space penalty factor.
-      // They are removed when the final tokens are generated.
+      // We add single space separator as prefixes of the terms that we extract.
+      // This information is needed to compute the space penalty factor of each term.
+      // These whitespace prefixes are removed when the final tokens are generated, or
+      // added as separated tokens when discardPunctuation is unset.
       if (Character.getType(buffer.get(pos)) == Character.SPACE_SEPARATOR) {
-        int nextChar = buffer.get(++pos);
-        while (nextChar != -1 && Character.getType(nextChar) == Character.SPACE_SEPARATOR) {
-          pos ++;
-          nextChar = buffer.get(pos);
+        if (buffer.get(++pos) == -1) {
+          pos = posData.pos;
         }
       }
-      if (buffer.get(pos) == -1) {
-        pos = posData.pos;
-      }
 
       boolean anyMatches = false;
 
@@ -651,7 +686,11 @@ public final class KoreanTokenizer extends Tokenizer {
       if (userFST != null) {
         userFST.getFirstArc(arc);
         int output = 0;
-        for(int posAhead=pos;;posAhead++) {
+        int maxPosAhead = 0;
+        int outputMaxPosAhead = 0;
+        int arcFinalOutMaxPosAhead = 0;
+
+        for (int posAhead = pos; ; posAhead++) {
           final int ch = buffer.get(posAhead);
           if (ch == -1) {
             break;
@@ -659,15 +698,23 @@ public final class KoreanTokenizer extends Tokenizer {
           if (userFST.findTargetArc(ch, arc, arc, posAhead == pos, userFSTReader) == null) {
             break;
           }
-          output += arc.output.intValue();
+          output += arc.output().intValue();
           if (arc.isFinal()) {
-            if (VERBOSE) {
-              System.out.println("    USER word " + new String(buffer.get(pos, posAhead - pos + 1)) + " toPos=" + (posAhead + 1));
-            }
-            add(userDictionary, posData, pos, posAhead+1, output + arc.nextFinalOutput.intValue(), Type.USER);
+            maxPosAhead = posAhead;
+            outputMaxPosAhead = output;
+            arcFinalOutMaxPosAhead = arc.nextFinalOutput().intValue();
             anyMatches = true;
           }
         }
+
+        // Longest matching for user word
+        if (anyMatches && maxPosAhead > userWordMaxPosAhead) {
+          if (VERBOSE) {
+            System.out.println("    USER word " + new String(buffer.get(pos, maxPosAhead + 1)) + " toPos=" + (maxPosAhead + 1));
+          }
+          add(userDictionary, posData, pos, maxPosAhead + 1, outputMaxPosAhead + arcFinalOutMaxPosAhead, Type.USER);
+          userWordMaxPosAhead = Math.max(userWordMaxPosAhead, maxPosAhead);
+        }
       }
 
       // TODO: we can be more aggressive about user
@@ -679,7 +726,7 @@ public final class KoreanTokenizer extends Tokenizer {
         fst.getFirstArc(arc);
         int output = 0;
 
-        for(int posAhead=pos;;posAhead++) {
+        for (int posAhead = pos; ; posAhead++) {
           final int ch = buffer.get(posAhead);
           if (ch == -1) {
             break;
@@ -690,7 +737,7 @@ public final class KoreanTokenizer extends Tokenizer {
             break;
           }
 
-          output += arc.output.intValue();
+          output += arc.output().intValue();
 
           // Optimization: for known words that are too-long
           // (compound), we should pre-compute the 2nd
@@ -699,12 +746,12 @@ public final class KoreanTokenizer extends Tokenizer {
           // match is found.
 
           if (arc.isFinal()) {
-            dictionary.lookupWordIds(output + arc.nextFinalOutput.intValue(), wordIdRef);
+            dictionary.lookupWordIds(output + arc.nextFinalOutput().intValue(), wordIdRef);
             if (VERBOSE) {
               System.out.println("    KNOWN word " + new String(buffer.get(pos, posAhead - pos + 1)) + " toPos=" + (posAhead + 1) + " " + wordIdRef.length + " wordIDs");
             }
             for (int ofs = 0; ofs < wordIdRef.length; ofs++) {
-              add(dictionary, posData, pos, posAhead+1, wordIdRef.ints[wordIdRef.offset + ofs], Type.KNOWN);
+              add(dictionary, posData, pos, posAhead + 1, wordIdRef.ints[wordIdRef.offset + ofs], Type.KNOWN);
               anyMatches = true;
             }
           }
@@ -728,8 +775,9 @@ public final class KoreanTokenizer extends Tokenizer {
         } else {
           // Extract unknown word. Characters with the same script are considered to be part of unknown word
           unknownWordLength = 1;
-          UnicodeScript scriptCode = UnicodeScript.of((int) firstCharacter);
+          UnicodeScript scriptCode = UnicodeScript.of(firstCharacter);
           final boolean isPunct = isPunctuation(firstCharacter);
+          final boolean isDigit = Character.isDigit(firstCharacter);
           for (int posAhead = pos + 1; unknownWordLength < MAX_UNKNOWN_WORD_LENGTH; posAhead++) {
             int next = buffer.get(posAhead);
             if (next == -1) {
@@ -744,7 +792,10 @@ public final class KoreanTokenizer extends Tokenizer {
                 || chType == Character.NON_SPACING_MARK;
 
             if (sameScript
+                  // split on punctuation
                   && isPunctuation(ch, chType) == isPunct
+                  // split on digit
+                  && Character.isDigit(ch) == isDigit
                   && characterDefinition.isGroup(ch)) {
               unknownWordLength++;
             } else {
@@ -859,11 +910,9 @@ public final class KoreanTokenizer extends Tokenizer {
               backWordPos+i,
               backWordPos+i+charLen
           );
-          if (shouldFilterToken(token) == false) {
-            pending.add(token);
-            if (VERBOSE) {
-              System.out.println("    add token=" + pending.get(pending.size() - 1));
-            }
+          pending.add(token);
+          if (VERBOSE) {
+            System.out.println("    add token=" + pending.get(pending.size() - 1));
           }
         }
       } else {
@@ -924,6 +973,16 @@ public final class KoreanTokenizer extends Tokenizer {
           }
         }
       }
+      if (discardPunctuation == false && backWordPos != backPos) {
+        // Add a token for whitespaces between terms
+        int offset = backPos - lastBackTracePos;
+        int len = backWordPos - backPos;
+        //System.out.println(offset + " " + fragmentOffset + " " + len + " " + backWordPos + " " + backPos);
+        unkDictionary.lookupWordIds(characterDefinition.getCharacterClass(' '), wordIdRef);
+        DictionaryToken spaceToken = new DictionaryToken(Type.UNKNOWN, unkDictionary,
+            wordIdRef.ints[wordIdRef.offset], fragment, offset, len, backPos, backPos+len);
+        pending.add(spaceToken);
+      }
 
       pos = backPos;
       bestIDX = nextBestIDX;
@@ -945,7 +1004,7 @@ public final class KoreanTokenizer extends Tokenizer {
   }
 
   private boolean shouldFilterToken(Token token) {
-    return isPunctuation(token.getSurfaceForm()[token.getOffset()]);
+    return discardPunctuation && isPunctuation(token.getSurfaceForm()[token.getOffset()]);
   }
 
   private static boolean isPunctuation(char ch) {
