@@ -97,9 +97,7 @@ final class OrdsSegmentTermsEnumFrame {
 
   final BlockTermState state;
 
-  // metadata buffer, holding monotonic values
-  public long[] longs;
-  // metadata buffer, holding general values
+  // metadata
   public byte[] bytes;
   ByteArrayDataInput bytesReader;
 
@@ -110,7 +108,6 @@ final class OrdsSegmentTermsEnumFrame {
     this.ord = ord;
     this.state = ste.fr.parent.postingsReader.newTermState();
     this.state.totalTermFreq = -1;
-    this.longs = new long[ste.fr.longsSize];
   }
 
   public void setFloorData(ByteArrayDataInput in, BytesRef source) {
@@ -499,17 +496,16 @@ final class OrdsSegmentTermsEnumFrame {
       // stats
       state.docFreq = statsReader.readVInt();
       //if (DEBUG) System.out.println("    dF=" + state.docFreq);
-      if (ste.fr.fieldInfo.getIndexOptions() != IndexOptions.DOCS) {
+      if (ste.fr.fieldInfo.getIndexOptions() == IndexOptions.DOCS) {
+        state.totalTermFreq = state.docFreq; // all tf values are 1
+      } else {
         state.totalTermFreq = state.docFreq + statsReader.readVLong();
         //if (DEBUG) System.out.println("    totTF=" + state.totalTermFreq);
       }
       //if (DEBUG) System.out.println("    longsSize=" + ste.fr.longsSize);
 
-      // metadata 
-      for (int i = 0; i < ste.fr.longsSize; i++) {
-        longs[i] = bytesReader.readVLong();
-      }
-      ste.fr.parent.postingsReader.decodeTerm(longs, bytesReader, ste.fr.fieldInfo, state, absolute);
+      // metadata
+      ste.fr.parent.postingsReader.decodeTerm(bytesReader, ste.fr.fieldInfo, state, absolute);
 
       metaDataUpto++;
       absolute = false;
