@@ -1209,14 +1209,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
     // Test1: test that leafReaders are sorted according to leafSorter provided in IndexWriterConfig
     {
       try (DirectoryReader reader = writer.getReader()) {
-        List<LeafReader> lrs =
-            reader.leaves().stream().map(LeafReaderContext::reader).collect(toList());
-        List<LeafReader> expectedSortedlrs =
-            reader.leaves().stream()
-                .map(LeafReaderContext::reader)
-                .sorted(leafSorter)
-                .collect(toList());
-        assertEquals(expectedSortedlrs, lrs);
+        assertLeavesSorted(reader, leafSorter);
 
         // add more documents that should be sorted first
         final long FIRST_VALUE = ASC_SORT ? 0 : 100;
@@ -1229,28 +1222,16 @@ public class TestIndexWriterReader extends LuceneTestCase {
 
         // and open again
         try (DirectoryReader reader2 = DirectoryReader.openIfChanged(reader)) {
-          lrs = reader2.leaves().stream().map(LeafReaderContext::reader).collect(toList());
-          expectedSortedlrs =
-              reader2.leaves().stream()
-                  .map(LeafReaderContext::reader)
-                  .sorted(leafSorter)
-                  .collect(toList());
-          assertEquals(expectedSortedlrs, lrs);
+          assertLeavesSorted(reader2, leafSorter);
         }
       }
     }
 
-    // Test2: test that leafReaders are sorted according to leafSorter provided in DirectoryReader
+    // Test2: test that leafReaders are sorted according to the provided leafSorter when opened from
+    // directory
     {
       try (DirectoryReader reader = DirectoryReader.open(dir, leafSorter)) {
-        List<LeafReader> lrs =
-            reader.leaves().stream().map(LeafReaderContext::reader).collect(toList());
-        List<LeafReader> expectedSortedlrs =
-            reader.leaves().stream()
-                .map(LeafReaderContext::reader)
-                .sorted(leafSorter)
-                .collect(toList());
-        assertEquals(expectedSortedlrs, lrs);
+        assertLeavesSorted(reader, leafSorter);
 
         // add more documents that should be sorted first
         final long FIRST_VALUE = ASC_SORT ? 0 : 100;
@@ -1263,13 +1244,7 @@ public class TestIndexWriterReader extends LuceneTestCase {
 
         // and open again
         try (DirectoryReader reader2 = DirectoryReader.openIfChanged(reader)) {
-          lrs = reader2.leaves().stream().map(LeafReaderContext::reader).collect(toList());
-          expectedSortedlrs =
-              reader2.leaves().stream()
-                  .map(LeafReaderContext::reader)
-                  .sorted(leafSorter)
-                  .collect(toList());
-          assertEquals(expectedSortedlrs, lrs);
+          assertLeavesSorted(reader2, leafSorter);
         }
       }
     }
@@ -1278,15 +1253,8 @@ public class TestIndexWriterReader extends LuceneTestCase {
     // to leafSorter of its wrapped reader
     {
       try (DirectoryReader reader =
-               new AssertingDirectoryReader(DirectoryReader.open(dir, leafSorter))) {
-        List<LeafReader> lrs =
-            reader.leaves().stream().map(LeafReaderContext::reader).collect(toList());
-        List<LeafReader> expectedSortedlrs =
-            reader.leaves().stream()
-                .map(LeafReaderContext::reader)
-                .sorted(leafSorter)
-                .collect(toList());
-        assertEquals(expectedSortedlrs, lrs);
+                   new AssertingDirectoryReader(DirectoryReader.open(dir, leafSorter))) {
+        assertLeavesSorted(reader, leafSorter);
 
         // add more documents that should be sorted first
         final long FIRST_VALUE = ASC_SORT ? 0 : 100;
@@ -1299,13 +1267,32 @@ public class TestIndexWriterReader extends LuceneTestCase {
 
         // and open again
         try (DirectoryReader reader2 = DirectoryReader.openIfChanged(reader)) {
-          lrs = reader2.leaves().stream().map(LeafReaderContext::reader).collect(toList());
-          expectedSortedlrs =
-              reader2.leaves().stream()
-                  .map(LeafReaderContext::reader)
-                  .sorted(leafSorter)
-                  .collect(toList());
-          assertEquals(expectedSortedlrs, lrs);
+          assertLeavesSorted(reader2, leafSorter);
+        }
+      }
+    }
+
+    // Test4: test that leafReaders are sorted according to the provided leafSorter when opened from
+    // commit
+    {
+      List<IndexCommit> commits = DirectoryReader.listCommits(dir);
+      IndexCommit latestCommit = commits.get(commits.size() - 1);
+      try (DirectoryReader reader =
+                   DirectoryReader.open(latestCommit, leafSorter)) {
+        assertLeavesSorted(reader, leafSorter);
+
+        // add more documents that should be sorted first
+        final long FIRST_VALUE = ASC_SORT ? 0 : 100;
+        for (int i = 0; i < 10; ++i) {
+          final Document doc = new Document();
+          doc.add(new LongPoint(FIELD_NAME, FIRST_VALUE));
+          writer.addDocument(doc);
+        }
+        writer.commit();
+
+        // and open again
+        try (DirectoryReader reader2 = DirectoryReader.openIfChanged(reader)) {
+          assertLeavesSorted(reader2, leafSorter);
         }
       }
     }
@@ -1313,4 +1300,18 @@ public class TestIndexWriterReader extends LuceneTestCase {
     writer.close();
     dir.close();
   }
+
+  // assert that the leaf readers of the provided directory reader are sorted according to the
+  // provided leafSorter
+  private static void assertLeavesSorted(
+          DirectoryReader reader, Comparator<LeafReader> leafSorter) {
+    List<LeafReader> lrs =
+            reader.leaves().stream().map(LeafReaderContext::reader).collect(toList());
+    List<LeafReader> expectedSortedlrs =
+            reader.leaves().stream()
+                    .map(LeafReaderContext::reader)
+                    .sorted(leafSorter)
+                    .collect(toList());
+    assertEquals(expectedSortedlrs, lrs);
+  }
 }
