@@ -29,6 +29,7 @@ import java.util.concurrent.TimeUnit;
 
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
+import org.apache.lucene.document.StringField;
 import org.apache.lucene.facet.DrillDownQuery;
 import org.apache.lucene.facet.FacetResult;
 import org.apache.lucene.facet.FacetTestCase;
@@ -46,6 +47,7 @@ import org.apache.lucene.search.TopDocs;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.util.Accountable;
 import org.apache.lucene.util.IOUtils;
+import org.apache.lucene.util.LuceneTestCase;
 import org.apache.lucene.util.NamedThreadFactory;
 import org.apache.lucene.util.TestUtil;
 
@@ -101,6 +103,111 @@ public class TestSortedSetDocValuesFacets extends FacetTestCase {
     IOUtils.close(searcher.getIndexReader(), dir);
   }
 
+  // See: LUCENE-10070
+  public void testCountAll() throws Exception {
+    Directory dir = newDirectory();
+
+    FacetsConfig config = new FacetsConfig();
+    RandomIndexWriter writer = new RandomIndexWriter(random(), dir);
+
+    Document doc = new Document();
+    doc.add(new StringField("id", "0", Field.Store.NO));
+    doc.add(new SortedSetDocValuesFacetField("a", "foo"));
+    writer.addDocument(config.build(doc));
+
+    doc = new Document();
+    doc.add(new StringField("id", "1", Field.Store.NO));
+    doc.add(new SortedSetDocValuesFacetField("a", "bar"));
+    writer.addDocument(config.build(doc));
+
+    writer.deleteDocuments(new Term("id", "0"));
+
+    // NRT open
+    IndexSearcher searcher = newSearcher(writer.getReader());
+
+    // Per-top-reader state:
+    SortedSetDocValuesReaderState state =
+        new DefaultSortedSetDocValuesReaderState(searcher.getIndexReader());
+
+    Facets facets = new SortedSetDocValuesFacetCounts(state);
+
+    assertEquals(
+        "dim=a path=[] value=1 childCount=1\n  bar (1)\n",
+        facets.getTopChildren(10, "a").toString());
+
+    ExecutorService exec =
+        new ThreadPoolExecutor(
+            1,
+            TestUtil.nextInt(random(), 2, 6),
+            Long.MAX_VALUE,
+            TimeUnit.MILLISECONDS,
+            new LinkedBlockingQueue<Runnable>(),
+            new NamedThreadFactory("TestIndexSearcher"));
+
+    facets = new ConcurrentSortedSetDocValuesFacetCounts(state, exec);
+
+    assertEquals(
+        "dim=a path=[] value=1 childCount=1\n  bar (1)\n",
+        facets.getTopChildren(10, "a").toString());
+
+    writer.close();
+    IOUtils.close(searcher.getIndexReader(), dir);
+    exec.shutdownNow();
+  }
+
+  public void testBasicSingleValued() throws Exception {
+    Directory dir = newDirectory();
+
+    FacetsConfig config = new FacetsConfig();
+    config.setMultiValued("a", false);
+    RandomIndexWriter writer = new RandomIndexWriter(random(), dir);
+
+    Document doc = new Document();
+    doc.add(new SortedSetDocValuesFacetField("a", "foo"));
+    doc.add(new SortedSetDocValuesFacetField("b", "bar"));
+    writer.addDocument(config.build(doc));
+    doc = new Document();
+    doc.add(new SortedSetDocValuesFacetField("a", "foo"));
+    writer.addDocument(config.build(doc));
+    if (random().nextBoolean()) {
+      writer.commit();
+    }
+
+    doc = new Document();
+    doc.add(new SortedSetDocValuesFacetField("a", "baz"));
+    writer.addDocument(config.build(doc));
+
+    // NRT open
+    IndexSearcher searcher = newSearcher(writer.getReader());
+
+    // Per-top-reader state:
+    SortedSetDocValuesReaderState state =
+        new DefaultSortedSetDocValuesReaderState(searcher.getIndexReader());
+
+    ExecutorService exec = randomExecutorServiceOrNull();
+    Facets facets = getAllFacets(searcher, state, exec);
+
+    assertEquals(
+        "dim=a path=[] value=3 childCount=2\n  foo (2)\n  baz (1)\n",
+        facets.getTopChildren(10, "a").toString());
+    assertEquals(
+        "dim=b path=[] value=1 childCount=1\n  bar (1)\n",
+        facets.getTopChildren(10, "b").toString());
+
+    // DrillDown:
+    DrillDownQuery q = new DrillDownQuery(config);
+    q.add("a", "foo");
+    q.add("b", "bar");
+    TopDocs hits = searcher.search(q, 1);
+    assertEquals(1, hits.totalHits.value);
+
+    if (exec != null) {
+      exec.shutdownNow();
+    }
+    writer.close();
+    IOUtils.close(searcher.getIndexReader(), dir);
+  }
+
   public void testDrillDownOptions() throws Exception {
     Directory dir = newDirectory();
 
@@ -339,99 +446,109 @@ public class TestSortedSetDocValuesFacets extends FacetTestCase {
   }
 
   public void testRandom() throws Exception {
-    String[] tokens = getRandomTokens(10);
-    Directory indexDir = newDirectory();
-    Directory taxoDir = newDirectory();
+    int fullIterations = LuceneTestCase.TEST_NIGHTLY ? 20 : 3;
+    for (int fullIter = 0; fullIter < fullIterations; fullIter++) {
+      String[] tokens = getRandomTokens(10);
+      Directory indexDir = newDirectory();
+      Directory taxoDir = newDirectory();
 
-    RandomIndexWriter w = new RandomIndexWriter(random(), indexDir);
-    FacetsConfig config = new FacetsConfig();
-    int numDocs = atLeast(1000);
-    int numDims = TestUtil.nextInt(random(), 1, 7);
-    List<TestDoc> testDocs = getRandomDocs(tokens, numDocs, numDims);
-    for(TestDoc testDoc : testDocs) {
-      Document doc = new Document();
-      doc.add(newStringField("content", testDoc.content, Field.Store.NO));
-      for(int j=0;j<numDims;j++) {
-        if (testDoc.dims[j] != null) {
-          doc.add(new SortedSetDocValuesFacetField("dim" + j, testDoc.dims[j]));
-        }
-      }
-      w.addDocument(config.build(doc));
-    }
-
-    // NRT open
-    IndexSearcher searcher = newSearcher(w.getReader());
-    
-    // Per-top-reader state:
-    SortedSetDocValuesReaderState state = new DefaultSortedSetDocValuesReaderState(searcher.getIndexReader());
-    ExecutorService exec = randomExecutorServiceOrNull();
-
-    int iters = atLeast(100);
-    for(int iter=0;iter<iters;iter++) {
-      String searchToken = tokens[random().nextInt(tokens.length)];
-      if (VERBOSE) {
-        System.out.println("\nTEST: iter content=" + searchToken);
-      }
-      FacetsCollector fc = new FacetsCollector();
-      FacetsCollector.search(searcher, new TermQuery(new Term("content", searchToken)), 10, fc);
-      Facets facets;
-      if (exec != null) {
-        facets = new ConcurrentSortedSetDocValuesFacetCounts(state, fc, exec);
+      RandomIndexWriter w = new RandomIndexWriter(random(), indexDir);
+      FacetsConfig config = new FacetsConfig();
+      int numDocs = atLeast(1000);
+      // Most of the time allow up to 7 dims per doc, but occasionally limit all docs to a single
+      // dim:
+      int numDims;
+      if (random().nextInt(10) < 8) {
+        numDims = TestUtil.nextInt(random(), 1, 7);
       } else {
-        facets = new SortedSetDocValuesFacetCounts(state, fc);
+        numDims = 1;
+      }
+      List<TestDoc> testDocs = getRandomDocs(tokens, numDocs, numDims);
+      for (TestDoc testDoc : testDocs) {
+        Document doc = new Document();
+        doc.add(newStringField("content", testDoc.content, Field.Store.NO));
+        for (int j = 0; j < numDims; j++) {
+          if (testDoc.dims[j] != null) {
+            doc.add(new SortedSetDocValuesFacetField("dim" + j, testDoc.dims[j]));
+          }
+        }
+        w.addDocument(config.build(doc));
       }
 
-      // Slow, yet hopefully bug-free, faceting:
-      @SuppressWarnings({"rawtypes","unchecked"}) Map<String,Integer>[] expectedCounts = new HashMap[numDims];
-      for(int i=0;i<numDims;i++) {
-        expectedCounts[i] = new HashMap<>();
-      }
+      // NRT open
+      IndexSearcher searcher = newSearcher(w.getReader());
 
-      for(TestDoc doc : testDocs) {
-        if (doc.content.equals(searchToken)) {
-          for(int j=0;j<numDims;j++) {
-            if (doc.dims[j] != null) {
-              Integer v = expectedCounts[j].get(doc.dims[j]);
-              if (v == null) {
-                expectedCounts[j].put(doc.dims[j], 1);
-              } else {
-                expectedCounts[j].put(doc.dims[j], v.intValue() + 1);
+      // Per-top-reader state:
+      SortedSetDocValuesReaderState state = new DefaultSortedSetDocValuesReaderState(searcher.getIndexReader());
+      ExecutorService exec = randomExecutorServiceOrNull();
+
+      int iters = atLeast(100);
+      for (int iter = 0; iter < iters; iter++) {
+        String searchToken = tokens[random().nextInt(tokens.length)];
+        if (VERBOSE) {
+          System.out.println("\nTEST: iter content=" + searchToken);
+        }
+        FacetsCollector fc = new FacetsCollector();
+        FacetsCollector.search(searcher, new TermQuery(new Term("content", searchToken)), 10, fc);
+        Facets facets;
+        if (exec != null) {
+          facets = new ConcurrentSortedSetDocValuesFacetCounts(state, fc, exec);
+        } else {
+          facets = new SortedSetDocValuesFacetCounts(state, fc);
+        }
+
+        // Slow, yet hopefully bug-free, faceting:
+        @SuppressWarnings({"rawtypes", "unchecked"}) Map<String, Integer>[] expectedCounts = new HashMap[numDims];
+        for (int i = 0; i < numDims; i++) {
+          expectedCounts[i] = new HashMap<>();
+        }
+
+        for (TestDoc doc : testDocs) {
+          if (doc.content.equals(searchToken)) {
+            for (int j = 0; j < numDims; j++) {
+              if (doc.dims[j] != null) {
+                Integer v = expectedCounts[j].get(doc.dims[j]);
+                if (v == null) {
+                  expectedCounts[j].put(doc.dims[j], 1);
+                } else {
+                  expectedCounts[j].put(doc.dims[j], v.intValue() + 1);
+                }
               }
             }
           }
         }
+
+        List<FacetResult> expected = new ArrayList<>();
+        for (int i = 0; i < numDims; i++) {
+          List<LabelAndValue> labelValues = new ArrayList<>();
+          int totCount = 0;
+          for (Map.Entry<String, Integer> ent : expectedCounts[i].entrySet()) {
+            labelValues.add(new LabelAndValue(ent.getKey(), ent.getValue()));
+            totCount += ent.getValue();
+          }
+          sortLabelValues(labelValues);
+          if (totCount > 0) {
+            expected.add(new FacetResult("dim" + i, new String[0], totCount, labelValues.toArray(new LabelAndValue[labelValues.size()]), labelValues.size()));
+          }
+        }
+
+        // Sort by highest value, tie break by value:
+        sortFacetResults(expected);
+
+        List<FacetResult> actual = facets.getAllDims(10);
+
+        // Messy: fixup ties
+        //sortTies(actual);
+
+        assertEquals(expected, actual);
       }
 
-      List<FacetResult> expected = new ArrayList<>();
-      for(int i=0;i<numDims;i++) {
-        List<LabelAndValue> labelValues = new ArrayList<>();
-        int totCount = 0;
-        for(Map.Entry<String,Integer> ent : expectedCounts[i].entrySet()) {
-          labelValues.add(new LabelAndValue(ent.getKey(), ent.getValue()));
-          totCount += ent.getValue();
-        }
-        sortLabelValues(labelValues);
-        if (totCount > 0) {
-          expected.add(new FacetResult("dim" + i, new String[0], totCount, labelValues.toArray(new LabelAndValue[labelValues.size()]), labelValues.size()));
-        }
+      if (exec != null) {
+        exec.shutdownNow();
       }
-
-      // Sort by highest value, tie break by value:
-      sortFacetResults(expected);
-
-      List<FacetResult> actual = facets.getAllDims(10);
-
-      // Messy: fixup ties
-      //sortTies(actual);
-
-      assertEquals(expected, actual);
+      w.close();
+      IOUtils.close(searcher.getIndexReader(), indexDir, taxoDir);
     }
-
-    if (exec != null) {
-      exec.shutdownNow();
-    }
-    w.close();
-    IOUtils.close(searcher.getIndexReader(), indexDir, taxoDir);
   }
 
   private static Facets getAllFacets(IndexSearcher searcher, SortedSetDocValuesReaderState state,
