@@ -83,8 +83,6 @@ import static org.hamcrest.CoreMatchers.equalTo;
 public class TestMemoryIndexAgainstRAMDir extends BaseTokenStreamTestCase {
   private Set<String> queries = new HashSet<>();
   
-  public static final int ITERATIONS = 100 * RANDOM_MULTIPLIER;
-
   @Override
   public void setUp() throws Exception {
     super.setUp();
@@ -115,7 +113,8 @@ public class TestMemoryIndexAgainstRAMDir extends BaseTokenStreamTestCase {
    */
   public void testRandomQueries() throws Exception {
     MemoryIndex index = randomMemoryIndex();
-    for (int i = 0; i < ITERATIONS; i++) {
+    int iterations = TEST_NIGHTLY ? 100 * RANDOM_MULTIPLIER : 10 * RANDOM_MULTIPLIER;
+    for (int i = 0; i < iterations; i++) {
       assertAgainstRAMDirectory(index);
     }
   }
@@ -171,7 +170,7 @@ public class TestMemoryIndexAgainstRAMDir extends BaseTokenStreamTestCase {
   private void duellReaders(CompositeReader other, LeafReader memIndexReader)
       throws IOException {
     Fields memFields = memIndexReader.getTermVectors(0);
-    for (String field : MultiFields.getFields(other)) {
+    for (String field : FieldInfos.getIndexedFields(other)) {
       Terms memTerms = memFields.terms(field);
       Terms iwTerms = memIndexReader.terms(field);
       if (iwTerms == null) {
@@ -246,7 +245,7 @@ public class TestMemoryIndexAgainstRAMDir extends BaseTokenStreamTestCase {
     for (String query : queries) {
       TopDocs ramDocs = ram.search(qp.parse(query), 1);
       TopDocs memDocs = mem.search(qp.parse(query), 1);
-      assertEquals(query, ramDocs.totalHits, memDocs.totalHits);
+      assertEquals(query, ramDocs.totalHits.value, memDocs.totalHits.value);
     }
     reader.close();
   }
@@ -652,7 +651,7 @@ public class TestMemoryIndexAgainstRAMDir extends BaseTokenStreamTestCase {
     memory.addField("foo", new CannedTokenStream(new Token("", 0, 5)));
     IndexSearcher searcher = memory.createSearcher();
     TopDocs docs = searcher.search(new TermQuery(new Term("foo", "")), 10);
-    assertEquals(1, docs.totalHits);
+    assertEquals(1, docs.totalHits.value);
     TestUtil.checkReader(searcher.getIndexReader());
   }
 
