@@ -19,6 +19,7 @@ package org.apache.lucene.replicator.nrt;
 
 import java.io.IOException;
 import java.io.PrintStream;
+import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -28,6 +29,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
@@ -42,7 +44,9 @@ import org.apache.lucene.search.SearcherFactory;
 import org.apache.lucene.search.TermQuery;
 import org.apache.lucene.store.AlreadyClosedException;
 import org.apache.lucene.store.BufferedChecksumIndexInput;
-import org.apache.lucene.store.ByteArrayIndexInput;
+import org.apache.lucene.store.ByteBuffersDataInput;
+import org.apache.lucene.store.ByteBuffersIndexInput;
+import org.apache.lucene.store.ChecksumIndexInput;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.store.IOContext;
 import org.apache.lucene.store.IndexOutput;
@@ -242,7 +246,7 @@ public abstract class ReplicaNode extends Node {
         byte[] infosBytes = job.getCopyState().infosBytes;
 
         SegmentInfos syncInfos = SegmentInfos.readCommit(dir,
-                                                         new BufferedChecksumIndexInput(new ByteArrayIndexInput("SegmentInfos", job.getCopyState().infosBytes)),
+                                                         toIndexInput(job.getCopyState().infosBytes),
                                                          job.getCopyState().gen);
 
         // Must always commit to a larger generation than what's currently in the index:
@@ -295,7 +299,7 @@ public abstract class ReplicaNode extends Node {
       message("top: done start");
       state = "idle";
     } catch (Throwable t) {
-      if (t.getMessage().startsWith("replica cannot start") == false) {
+      if (Objects.toString(t.getMessage()).startsWith("replica cannot start") == false) {
         message("exc on start:");
         t.printStackTrace(printStream);
       } else {
@@ -382,7 +386,7 @@ public abstract class ReplicaNode extends Node {
       // Turn byte[] back to SegmentInfos:
       byte[] infosBytes = copyState.infosBytes;
       SegmentInfos infos = SegmentInfos.readCommit(dir,
-                                                   new BufferedChecksumIndexInput(new ByteArrayIndexInput("SegmentInfos", copyState.infosBytes)),
+                                                   toIndexInput(copyState.infosBytes),
                                                    copyState.gen);
       assert infos.getVersion() == copyState.version;
 
@@ -439,6 +443,13 @@ public abstract class ReplicaNode extends Node {
                           markerCount));
   }
 
+  private ChecksumIndexInput toIndexInput(byte[] input) {
+    return new BufferedChecksumIndexInput(
+        new ByteBuffersIndexInput(
+            new ByteBuffersDataInput(
+                Arrays.asList(ByteBuffer.wrap(input))), "SegmentInfos"));
+  }
+
   /** Start a background copying job, to copy the specified files from the current primary node.  If files is null then the latest copy
    *  state should be copied.  If prevJob is not null, then the new copy job is replacing it and should 1) cancel the previous one, and
    *  2) optionally salvage e.g. partially copied and, shared with the new copy job, files. */
@@ -720,8 +731,8 @@ public abstract class ReplicaNode extends Node {
   }
 
   /** Carefully determine if the file on the primary, identified by its {@code String fileName} along with the {@link FileMetaData}
-   * "summarizing" its contents, is precisely the same file that we have locally.  If the file does not exist locally, or if its its header
-   * (inclues the segment id), length, footer (including checksum) differ, then this returns false, else true. */
+   * "summarizing" its contents, is precisely the same file that we have locally.  If the file does not exist locally, or if its header
+   * (includes the segment id), length, footer (including checksum) differ, then this returns false, else true. */
   private boolean fileIsIdentical(String fileName, FileMetaData srcMetaData) throws IOException {
 
     FileMetaData destMetaData = readLocalFileMetaData(fileName);
