@@ -66,6 +66,8 @@ public abstract class BaseFieldInfoFormatTestCase extends BaseIndexFileFormatTes
     assertFalse(infos2.fieldInfo("field").omitsNorms());
     assertFalse(infos2.fieldInfo("field").hasPayloads());
     assertFalse(infos2.fieldInfo("field").hasVectors());
+    assertEquals(0, infos2.fieldInfo("field").getPointDimensionCount());
+    assertFalse(infos2.fieldInfo("field").isSoftDeletesField());
     dir.close();
   }
   
@@ -242,7 +244,12 @@ public abstract class BaseFieldInfoFormatTestCase extends BaseIndexFileFormatTes
     for (int i = 0; i < numFields; i++) {
       fieldNames.add(TestUtil.randomUnicodeString(random()));
     }
-    FieldInfos.Builder builder = new FieldInfos.Builder(new FieldInfos.FieldNumbers(null));
+
+    String softDeletesField =
+        random().nextBoolean() ? TestUtil.randomUnicodeString(random()) : null;
+    FieldInfos.Builder builder =
+        new FieldInfos.Builder(new FieldInfos.FieldNumbers(softDeletesField));
+
     for (String field : fieldNames) {
       IndexableFieldType fieldType = randomFieldType(random());
       FieldInfo fi = builder.getOrAdd(field);
@@ -259,6 +266,14 @@ public abstract class BaseFieldInfoFormatTestCase extends BaseIndexFileFormatTes
           fi.setStorePayloads();
         }
       }
+
+      if (fieldType.pointDimensionCount() > 0) {
+        fi.setPointDimensions(
+            fieldType.pointDimensionCount(),
+            fieldType.pointIndexDimensionCount(),
+            fieldType.pointNumBytes());
+      }
+
       addAttributes(fi);
     }
     FieldInfos infos = builder.finish();
@@ -272,7 +287,7 @@ public abstract class BaseFieldInfoFormatTestCase extends BaseIndexFileFormatTes
     FieldType type = new FieldType();
     
     if (r.nextBoolean()) {
-      IndexOptions values[] = IndexOptions.values();
+      IndexOptions[] values = IndexOptions.values();
       type.setIndexOptions(values[r.nextInt(values.length)]);
       type.setOmitNorms(r.nextBoolean());
       
@@ -289,30 +304,23 @@ public abstract class BaseFieldInfoFormatTestCase extends BaseIndexFileFormatTes
     }
     
     if (r.nextBoolean()) {
-      DocValuesType values[] = getDocValuesTypes();
+      DocValuesType[] values = DocValuesType.values();
       type.setDocValuesType(values[r.nextInt(values.length)]);
     }
-        
+
+    if (r.nextBoolean()) {
+      int dimension = 1 + r.nextInt(PointValues.MAX_DIMENSIONS);
+      int indexDimension = 1 + r.nextInt(Math.min(dimension, PointValues.MAX_INDEX_DIMENSIONS));
+      int dimensionNumBytes = 1 + r.nextInt(PointValues.MAX_NUM_BYTES);
+      type.setDimensions(dimension, indexDimension, dimensionNumBytes);
+    }
+
     return type;
   }
-  
-  /** 
-   * Hook to add any codec attributes to fieldinfo
-   * instances added in this test.
-   */
-  protected void addAttributes(FieldInfo fi) {
-  }
-  
-  /** 
-   * Docvalues types to test. 
-   * @deprecated only for Only available to ancient codecs can 
-   * limit this to the subset of types they support.
-   */
-  @Deprecated
-  protected DocValuesType[] getDocValuesTypes() {
-    return DocValuesType.values();
-  }
-  
+
+  /** Hook to add any codec attributes to fieldinfo instances added in this test. */
+  protected void addAttributes(FieldInfo fi) {}
+
   /** equality for entirety of fieldinfos */
   protected void assertEquals(FieldInfos expected, FieldInfos actual) {
     assertEquals(expected.size(), actual.size());
