@@ -249,7 +249,7 @@ public class MockDirectoryWrapper extends BaseDirectoryWrapper {
     maybeYield();
     maybeThrowDeterministicException();
     if (crashed) {
-      throw new IOException("cannot rename after crash");
+      throw new IOException("cannot sync metadata after crash");
     }
     in.syncMetaData();
   }
@@ -272,17 +272,22 @@ public class MockDirectoryWrapper extends BaseDirectoryWrapper {
   }
 
   public synchronized void corruptUnknownFiles() throws IOException {
-
-    System.out.println("MDW: corrupt unknown files");
+    if (LuceneTestCase.VERBOSE) {
+      System.out.println("MDW: corrupt unknown files");
+    }
     Set<String> knownFiles = new HashSet<>();
     for(String fileName : listAll()) {
       if (fileName.startsWith(IndexFileNames.SEGMENTS)) {
-        System.out.println("MDW: read " + fileName + " to gather files it references");
+        if (LuceneTestCase.VERBOSE) {
+          System.out.println("MDW: read " + fileName + " to gather files it references");
+        }
         SegmentInfos infos;
         try {
           infos = SegmentInfos.readCommit(this, fileName);
         } catch (IOException ioe) {
-          System.out.println("MDW: exception reading segment infos " + fileName + "; files: " + Arrays.toString(listAll()));
+          if (LuceneTestCase.VERBOSE) {
+            System.out.println("MDW: exception reading segment infos " + fileName + "; files: " + Arrays.toString(listAll()));
+          }
           throw ioe;
         }
         knownFiles.addAll(infos.files(true));
@@ -859,13 +864,18 @@ public class MockDirectoryWrapper extends BaseDirectoryWrapper {
             System.out.println("\nNOTE: MockDirectoryWrapper: now run CheckIndex");
           } 
 
-          TestUtil.checkIndex(this, getCrossCheckTermVectorsOnClose(), true, null);
+          // Methods in MockDirectoryWrapper hold locks on this, which will cause deadlock when
+          // TestUtil#checkIndex checks segment concurrently using another thread, but making
+          // call back to synchronized methods such as MockDirectoryWrapper#fileLength.
+          // Hence passing concurrent = false to this method to turn off concurrent checks.
+          TestUtil.checkIndex(this, getCrossCheckTermVectorsOnClose(), true, false, null);
         }
           
         // TODO: factor this out / share w/ TestIW.assertNoUnreferencedFiles
         if (assertNoUnreferencedFilesOnClose) {
-          System.out.println("MDW: now assert no unref'd files at close");
-
+          if (LuceneTestCase.VERBOSE) {
+            System.out.println("MDW: now assert no unref'd files at close");
+          }
           // now look for unreferenced files: discount ones that we tried to delete but could not
           Set<String> allFiles = new HashSet<>(Arrays.asList(listAll()));
           String[] startFiles = allFiles.toArray(new String[0]);
